@@ -8,19 +8,23 @@
     :copyright: Copyright 2016-2023 by Martín Gaitán and others
     :license: BSD, see LICENSE for details.
 """
+from __future__ import annotations
 
 import codecs
+import errno
 import os
+import posixpath
 import re
 from hashlib import sha1
 from subprocess import PIPE, Popen
 from tempfile import _get_default_tempdir
+import uuid
 
-import posixpath
 import sphinx
 from docutils import nodes
 from docutils.parsers.rst import Directive, directives
 from docutils.statemachine import ViewList
+from sphinx.application import Sphinx
 from sphinx.locale import _
 from sphinx.util import logging
 from sphinx.util.i18n import search_image_for_language
@@ -71,6 +75,7 @@ class Mermaid(Directive):
         "alt": directives.unchanged,
         "align": align_spec,
         "caption": directives.unchanged,
+        "zoom": directives.unchanged,
     }
 
     def get_mm_code(self):
@@ -103,19 +108,27 @@ class Mermaid(Directive):
         else:
             # inline mermaid code
             mmcode = "\n".join(self.content)
-            if not mmcode.strip():
-                return [
-                    self.state_machine.reporter.warning(
-                        'Ignoring "mermaid" directive without content.',
-                        line=self.lineno,
-                    )
-                ]
         return mmcode
 
     def run(self):
+        mmcode = self.get_mm_code()
+        # mmcode is a list, so it's a system message, not content to be included in the
+        # document.
+        if not isinstance(mmcode, str):
+            return mmcode
 
+        # if mmcode is empty, ignore the directive.
+        if not mmcode.strip():
+            return [
+                self.state_machine.reporter.warning(
+                    'Ignoring "mermaid" directive without content.',
+                    line=self.lineno,
+                )
+            ]
+
+        # Wrap the mermaid code into a code node.
         node = mermaid()
-        node["code"] = self.get_mm_code()
+        node["code"] = mmcode
         node["options"] = {}
         if "alt" in self.options:
             node["alt"] = self.options["alt"]
@@ -123,6 +136,9 @@ class Mermaid(Directive):
             node["align"] = self.options["align"]
         if "inline" in self.options:
             node["inline"] = True
+        if "zoom" in self.options:
+            node["zoom"] = True
+            node["zoom_id"] = f"id-{uuid.uuid4()}"
 
         caption = self.options.get("caption")
         if caption:
@@ -217,18 +233,28 @@ def render_mm(self, code, options, _fmt, prefix="mermaid"):
 def _render_mm_html_raw(
     self, node, code, options, prefix="mermaid", imgcls=None, alt=None
 ):
-    if "align" in node:
+    if "align" in node and "zoom_id" in node:
+        tag_template = """<div align="{align}" id="{zoom_id}" class="mermaid align-{align}">
+            {code}
+        </div>
+        """
+    elif "align" in node and "zoom_id" not in node:
         tag_template = """<div align="{align}" class="mermaid align-{align}">
             {code}
         </div>
         """
+    elif "align" not in node and "zoom_id" in node:
+        tag_template = """<div id="{zoom_id}" class="mermaid">
+            {code}
+        </div>
+        """
     else:
         tag_template = """<div class="mermaid">
             {code}
         </div>"""
 
     self.body.append(
-        tag_template.format(align=node.get("align"), code=self.encode(code))
+        tag_template.format(align=node.get("align"), zoom_id=node.get("zoom_id"), code=self.encode(code))
     )
     raise nodes.SkipNode
 
@@ -293,7 +319,7 @@ def render_mm_latex(self, node, code, options, prefix="mermaid"):
         try:
             p = Popen(mm_args, stdout=PIPE, stdin=PIPE, stderr=PIPE)
         except OSError as err:
-            if err.errno != ENOENT:  # No such file or directory
+            if err.errno != errno.ENOENT:   # No such file or directory
                 raise
             logger.warning(
                 f"command {self.builder.config.mermaid_pdfcrop!r} cannot be run (needed to crop pdf), check the mermaid_cmd setting"
@@ -378,10 +404,20 @@ def man_visit_mermaid(self, node):
     raise nodes.SkipNode
 
 
-def install_js(app, *args):
-    # add required javascript
+def install_js(
+    app: Sphinx,
+    pagename,
+    templatename: str,
+    context: dict,
+    doctree: nodes.document | None,
+) -> None:
+    # Skip for pages without Mermaid diagrams
+    if doctree and not doctree.next_node(mermaid):
+        return
+
+    # Add required JavaScript
     if not app.config.mermaid_version:
-        _mermaid_js_url = None  # asummed is local
+        _mermaid_js_url = None  # assume it is local
     elif app.config.mermaid_version == "latest":
         _mermaid_js_url = "https://unpkg.com/mermaid/dist/mermaid.min.js"
     else:
@@ -396,6 +432,54 @@ def install_js(app, *args):
         )
         app.add_js_file(None, body=app.config.mermaid_init_js, priority=priority)
 
+    if app.config.mermaid_output_format == "raw":
+        if app.config.mermaid_d3_zoom:
+            _d3_js_url = "https://unpkg.com/d3/dist/d3.min.js"
+            _d3_js_script = """
+            window.addEventListener("load", function () {
+              var svgs = d3.selectAll(".mermaid svg");
+              svgs.each(function() {
+                var svg = d3.select(this);
+                svg.html("<g>" + svg.html() + "</g>");
+                var inner = svg.select("g");
+                var zoom = d3.zoom().on("zoom", function(event) {
+                  inner.attr("transform", event.transform);
+                });
+                svg.call(zoom);
+              });
+            });
+            """
+            app.add_js_file(_d3_js_url, priority=app.config.mermaid_js_priority)
+            app.add_js_file(None, body=_d3_js_script, priority=app.config.mermaid_js_priority)
+        elif doctree:
+            mermaid_nodes = doctree.findall(mermaid)
+            _d3_selector = ""
+            for mermaid_node in mermaid_nodes:
+                if "zoom_id" in mermaid_node:
+                    _zoom_id = mermaid_node["zoom_id"]
+                    if _d3_selector == "":
+                        _d3_selector += f".mermaid#{_zoom_id} svg"
+                    else:
+                        _d3_selector += f", .mermaid#{_zoom_id} svg"
+            if _d3_selector != "":
+                _d3_js_url = "https://unpkg.com/d3/dist/d3.min.js"
+                _d3_js_script = f"""
+                window.addEventListener("load", function () {{
+                  var svgs = d3.selectAll("{_d3_selector}");
+                  svgs.each(function() {{
+                    var svg = d3.select(this);
+                    svg.html("<g>" + svg.html() + "</g>");
+                    var inner = svg.select("g");
+                    var zoom = d3.zoom().on("zoom", function(event) {{
+                      inner.attr("transform", event.transform);
+                    }});
+                    svg.call(zoom);
+                  }});
+                }});
+                """
+                app.add_js_file(_d3_js_url, priority=app.config.mermaid_js_priority)
+                app.add_js_file(None, body=_d3_js_script, priority=app.config.mermaid_js_priority)
+
 
 def setup(app):
     app.add_node(
@@ -421,12 +505,13 @@ def setup(app):
     # thus it requires a different initialization code not yet supported. 
     # So the current latest version supported is this
     # Discussion: https://github.com/mermaid-js/mermaid/discussions/4148
-    app.add_config_value("mermaid_version", "9.4.0", "html")
+    app.add_config_value("mermaid_version", "10.2.0", "html")
     app.add_config_value("mermaid_js_priority", 500, "html")
     app.add_config_value("mermaid_init_js_priority", 500, "html")
     app.add_config_value(
         "mermaid_init_js", "mermaid.initialize({startOnLoad:true});", "html"
     )
+    app.add_config_value("mermaid_d3_zoom", False, "html")
     app.connect("html-page-context", install_js)
 
     return {"version": sphinx.__display_version__, "parallel_read_safe": True}
